#ifndef _SCSI_GENERIC_H
#define _SCSI_GENERIC_H

/*
   History:
    Started: Aug 9 by Lawrence Foard (entropy@world.std.com), to allow user 
     process control of SCSI devices.   [which year was that? - dpg]
    Development Sponsored by Killy Corp. NY NY
Original driver (sg.h):
*       Copyright (C) 1992 Lawrence Foard

Enhancements for portable SCSI transport library written by
	Heiko Eifeld and Jrg Schilling, Copyright (C) 1998 Heiko Eifeld, Jrg Schilling

	New features:
	- New ioctl() SG_GET_BUFSIZE allows to request the max DMA size.
        - the SCSI target, host and driver status are returned
          in the unused field pack_len in sg_header (maintaining its original
	  structure).
	- The size of the SCSI cdb may be set in cdb_len

2.x extensions to driver:
*       Copyright (C) 1998, 1999 Douglas Gilbert


    Modifications for scatter-gather + command queuing: 2.1.9 (990309)
    This version for later 2.1.x series and 2.2.x kernels
    D. P. Gilbert (dgilbert@interlog.com, dougg@triode.net.au)

    Changes since 2.1.7 (990227)
        - command queuing now "non-default" [back. compat. with cdparanoia]
        - Tighten access on some ioctls
    Changes since 2.1.8 (990303)
        - debug ">9" option dumps debug for _all_ active devices
        - increase allowable dma pool usage + increase minimum threshhold
        - pad out sg_scsi_id structure


    New features and changes:
        - per file descriptor (fd) write-read sequencing and command queues.
        - command queuing supported (SG_MAX_QUEUE is maximum per fd).
        - scatter-gather supported (allowing potentially megabyte transfers).
        - asynchronous notification support added (SIGPOLL, SIGIO) for
          read()s ( write()s should never block).
        - pack_id logic added so read() can be made to wait for a specific
          pack_id. 
        - uses memory > ISA_DMA_THRESHOLD if adapter allows it (e.g. a
          pci scsi adapter). Also copes with non-contiguous allocations.
        - this driver no longer uses a single SG_BIG_BUFF sized buffer
          obtained at driver/module init time. Rather it obtains a 
          SG_SCATTER_SZ buffer when a fd is open()ed and frees it at
          the corresponding release() (ie pr fd). Hence open() can return
          ENOMEM! If write() request > SG_SCATTER_SZ bytes for data then
          it can fail with ENOMEM as well (if so, scale back).
        - adds several ioctl calls, see ioctl section below.
        - SG_SCATTER_SZ's presence indicates this version of "sg" driver.
 
 Good documentation on the original "sg" device interface and usage can be
 found in the Linux HOWTO document: "SCSI Programming HOWTO" by Heiko
 Eissfeldt; last updated 7 May 1996.
********
JS: This document is outdated.

A program that wants to use user level SCSI transport should rather use
the portable SCSI transport library found in 'cdrecord'. Using this library
garantees that the application is not depending on special (temporary)
features of the sg driver which should be replaced by a decent driver
that uses an interface similar to other operating systems.

********
 I will add more info on using the
 extensions in this driver as required. A quick summary:
 An SG device is accessed by writing SCSI commands plus any associated 
 outgoing data to it; the resulting status codes and any incoming data
 is then obtained by a read call. The device can be opened O_NONBLOCK
 (non-blocking) and poll() used to monitor its progress. The device may be
 opened O_EXCL which excludes other "sg" users from this device (but not 
 "sd" or "sr" users). The buffer given to the write() call is made up as
 follows:
        - struct sg_header image (see below)
        - scsi command
        - data to be written to the device (if any)

 The buffer received from the corresponding read() call contains:
        - struct sg_header image (check results + sense_buffer)
        - data read back from device (if any)

 The given SCSI command has its LUN field overwritten by an internal value.

 Memory (RAM) is used within this driver for direct memory access (DMA)
 in transferring data to and from the SCSI device. The dreaded ENOMEM
 seems to be more prevalent under early 2.2.x kernels than under the
 2.0.x kernel series. For a given (large) transfer the memory obtained by
 this driver must be contiguous or scatter-gather must be used (if
 supported by the adapter). [Furthermore, ISA SCSI adapters can only use
 memory below the 16MB level on a i386.]
 This driver tries hard to find some suitable memory before admitting
 defeat and returning ENOMEM. All is not lost if application writers
 then back off the amount they are requesting (not done yet :-) ).
 This driver does the following:
   -  attempts to reserve a SG_SCATTER_SZ sized buffer on open(). The
      actual amount reserved is given by the SG_GET_RESERVED_SIZE ioctl().
   -  each write() needs to reserve a DMA buffer of the size of the
      data buffer indicated (excluding sg_header and command overhead).
      This buffer, depending on its size, adapter type (ISA or not) and
      the amount of memory available will be obtained from the kernel
      directly (kmalloc or get_free_pages) or the from the scsi mid-level
      dma pool (taking care not to exhaust it).
      If the buffer requested is > SG_SCATTER_SZ or memory is tight then
      scatter-gather will be used if supported by the adapter.
  -   write() will also attempt to use the buffer reserved on open()
      if it is large enough.

 The above strategy ensures that a write() can always depend on a buffer 
 of the size indicated by the SG_GET_RESERVED_SIZE ioctl() (which could be
 0, but at least the app knows things are tight in advance).
 Hence application writers can adopt quite aggressive strategies (e.g. 
 requesting 512KB) and scale them back in the face of ENOMEM errors.
 N.B. Queuing up commands also ties up kernel memory.

 More documentation can be found at www.netwinder.org/~dougg
*/

/*
 * ioctl's
 *
 * Note that all timeouts are multiples of of a clock tick (HZ).
 * Hz is usually 100 but it is 1024 on DEC alpha!
 */
#define SG_SET_TIMEOUT	0x2201	/* set timeout *(int *)arg==timeout in clock ticks*/
#define SG_GET_TIMEOUT	0x2202	/* get timeout return timeout in clock ticks */
#define SG_EMULATED_HOST 0x2203	/* true for emulated host adapter (ATAPI) */

/* Used to configure SCSI command transformation layer for ATAPI devices */
#define SG_SET_TRANSFORM 0x2204	/* Activate mod- sense/select and inquiry tranform. */
#define SG_GET_TRANSFORM 0x2205	/* Query mod- sense/select and inquiry tranformation */

#define SG_GET_BUFSIZE	0x2206	/* garanteed DMA size, return actual SG_BIG_BUFF */

/*
 * Some configuration constants.
 * Don't change SG_MAX_SENSE, it would break binary comparibility.
 */
#define SG_MAX_SENSE	16	/* Size of auto-request-sense buffer */
#define SG_DEFAULT_TIMEOUT (60*HZ) /* 1 minute timeout */
#define SG_DEFAULT_RETRIES 1


/* 
 An SG device is accessed by writing "packets" to it, the replies
 are then read using the read call. The same header is used for 
 reply, just ignore reply_len field.
*/
#ifdef	SG_GET_BUFSIZE
/*
 * This is the new version of the data structure.
 * It tries to make the best approach without breaking the old behaviour.
 * This version is intended to make 'sg' usable for developing SCSI user
 * applications until a better driver is included into Linux.
 * this version of the driver is still not able to return the amount of 
 * sense data that is required by CCS (18 bytes).
 *
 * The new version is granted to be present if SG_GET_BUFSIZE is defined.
 * The fields pack_len, reply_len and sense_len are used for different
 * purpose when sending / receiving packets.
 *
 * On return:
 * pack_len is the cmd_status						->DONE
 * reply_len is the actual DMA transfer count				->FUTURE
 * sense_len is the number of bytes acually returned in sense_buffer	->FUTURE
 *
 * Only old code that does not clear the unused 31 bits of the 
 * old data structure breaks with the new version of the driver.
 * *** IMPORTANT *** clean code must clear the sg_header before usage.
 */
struct sg_header {
/*
 * cmd_status contains:
 * driver_byte << 24 | host_byte << 16 | msg_byte << 8 | status_byte
 */
#define sg_cmd_status   pack_len
  int pack_len;			/* [i] length of incoming packet (including header) / [o] command status */
  int reply_len;		/* [i] maximum length of expected reply / [o] actual transfer count */
  int pack_id;			/* [i/o] id number of packet */
  int result;			/* [o] 0==ok, otherwise refer to errno codes */
  unsigned int twelve_byte :1;	/* [i] OBSOLETE: Force 12 byte command length (unused if want_new is set) */
  unsigned int want_new    :1;	/* [i] User requests new behaviour */
  unsigned int grant_new   :1;	/* [o] Driver grants new behaviour */
  unsigned int cdb_len     :5;	/* [i] Command descriptor block length 6..31 (mandatory) */
  unsigned int sense_len   :5;	/* [i/o] Set max / get actual sense length 0..31 (mandatory) */
  unsigned int other_flags :19;	/* for future use */
  unsigned char sense_buffer[SG_MAX_SENSE];/* [o] used only by reads */
				/* XXX mid-level currently only allocates 16 bytes to sense data,
				 * XXX this violates CCS, SCSI-2, SCSI-3, but upograding would
				 * XXX break sg binary compatibility. If the mid-level is corrected,
				 * XXX we need to introduce a ioctl to read the sense data.
				 */
  /* command follows then data for command */
 };
#else
/*
 * This is the old version of the data structure it is included to
 * help developers to migrate to the new interface.
 */
struct sg_header
 {
  int pack_len;    /* length of incoming packet <4096 (including header) */
  int reply_len;   /* maximum length <4096 of expected reply */
  int pack_id;     /* id number of packet */
  int result;      /* 0==ok, otherwise refer to errno codes */
  unsigned int twelve_byte:1; /* Force 12 byte command length for group 6 & 7 commands  */
  unsigned int other_flags:31;			/* for future use */
  unsigned char sense_buffer[16]; /* used only by reads */
  /* command follows then data for command */
 };
#endif


typedef struct sg_scsi_id {
    int host_no;        /* as in "scsi<n>" where 'n' is one of 0, 1, 2 etc */
    int channel;
    int scsi_id;        /* scsi id of target device */
    int lun;
    int scsi_type;      /* TYPE_... defined in scsi/scsi.h */
    int unused1;        /* probably find a good use, set 0 for now */
    int unused2;        /* ditto */
    int unused3;  
} Sg_scsi_id;

/* ioctls  ( _GET_s yield result via 'int *' 3rd argument unless 
            otherwise indicated */

/* The following ioctl takes a 'Sg_scsi_id' object as its 3rd argument. */
#define SG_GET_SCSI_ID 0x2276   /* Yields fd's bus,chan,dev,lun+type */
/* SCSI id information can also be obtained from SCSI_IOCTL_GET_IDLUN */

/* Override adapter setting and alway DMA using low memory ( <16MB on i386).
   Default is 0 (off - use adapter setting) */
#define SG_SET_FORCE_LOW_DMA 0x2279  /* 0-> use adapter setting, 1-> force */
#define SG_GET_LOW_DMA 0x227a   /* 0-> use all ram for dma; 1-> low dma ram */

/* When SG_SET_FORCE_PACK_ID set to 1, pack_id is input to read() which
   will attempt to read that pack_id or block (or return EAGAIN). If 
   pack_id is -1 then read oldest waiting. When ...FORCE_PACK_ID set to 0
   (default) then pack_id ignored by read() and oldest readable fetched. */ 
#define SG_SET_FORCE_PACK_ID 0x227b
#define SG_GET_PACK_ID 0x227c /* Yields oldest readable pack_id (or -1) */

#define SG_GET_NUM_WAITING 0x227d /* Number of commands awaiting read() */

/* Turn on error sense trace (1..8), dump this device to log/console (9)
   or dump all sg device states ( >9 ) to log/console */
#define SG_SET_DEBUG 0x227e    /* 0 -> turn off debug */

/* Yields max scatter gather tablesize allowed by current host adapter */
#define SG_GET_SG_TABLESIZE 0x227F

/* Yields buffer size allocated on open(): PAGE_SIZE <= x <= SG_SCATTER_SZ */
#define SG_GET_RESERVED_SIZE 0x2272

/* Control whether sequencing per file descriptor (default) or per device */
#define SG_GET_MERGE_FD 0x2274   /* 0-> per fd (default), 1-> per device */
#define SG_SET_MERGE_FD 0x2273   /* Attempt to change sequencing state,
  if more than 1 fd open on device, will fail with EBUSY */

/* Get/set command queuing state per fd (default is SG_DEF_COMMAND_Q) */
#define SG_GET_COMMAND_Q 0x2270   /* Yields 0 (queuing off) or 1 (on) */
#define SG_SET_COMMAND_Q 0x2271   /* Change queuing state with 0 or 1 */

/* Command queuing is default on (1) or off (0), can be overridden per
   file descriptor with SG_SET_COMMAND_Q ioctl */
#define SG_DEF_COMMAND_Q 0
#define SG_DEF_MERGE_FD 0
#define SG_DEF_FORCE_LOW_DMA 0
#define SG_DEF_FORCE_PACK_ID 0

/* maximum outstanding requests, write() yields EDOM if exceeded */
#define SG_MAX_QUEUE 16

#define SG_SCATTER_SZ (8 * 4096)  /* PAGE_SIZE not available to user */
/* Largest size (in bytes) a single scatter-gather list element can have.
   The value must be a power of 2 and <= (PAGE_SIZE * 32) [131072 bytes on 
   i386]. The minimum value is PAGE_SIZE. If scatter-gather not supported
   by adapter then this value is the largest data block that can be
   read/written by a single scsi command. Max number of scatter-gather
   list elements seems to be limited to 255. */

#define SG_BIG_BUFF SG_SCATTER_SZ       /* for backward compatibility */
/* #define SG_BIG_BUFF (SG_SCATTER_SZ * 8) */ /* =256KB, if you want */

#endif
